@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
 from .attr_symbols import AttrSubSymbolChain, CallPoint, get_attrsub_symbol_chain
 from .lineno_stmt_map import compute_lineno_to_stmt_mapping
-from .reachable_refs import *
+from .live_refs import *
 from .stmt_edges import *
