@@ -9,6 +9,11 @@ if TYPE_CHECKING:
     from typing import Sequence, Union
 
 
+class TiedTuple(tuple):
+    """Just a marker class indicating that we should not unpack contents of this tuple"""
+    pass
+
+
 class GetAssignmentLvalRvalSymbolRefs(SaveOffAttributesMixin, VisitListsMixin, ast.NodeVisitor):
     def __init__(self):
         # TODO: figure out how to give these type annotations
@@ -58,25 +63,28 @@ class GetAssignmentLvalRvalSymbolRefs(SaveOffAttributesMixin, VisitListsMixin, a
         self.visit_List_or_Tuple(node)
 
     def visit_Dict(self, node):
-        inner_symbols = []
-        with self.push_attributes(to_add_set=inner_symbols):
-            self.visit(node.keys)
-            self.visit(node.values)
-        self.to_add_set.append(tuple(inner_symbols))
+        temp = self.to_add_set
+        self.to_add_set = []
+        self.visit(node.keys)
+        self.visit(node.values)
+        self.to_add_set, temp = temp, self.to_add_set
+        self.to_add_set.append(tuple(temp))
 
     def visit_List_or_Tuple(self, node):
-        inner_symbols = []
-        with self.push_attributes(to_add_set=inner_symbols):
-            self.visit(node.elts)
-        self.to_add_set.append(tuple(inner_symbols))
+        temp = self.to_add_set
+        self.to_add_set = []
+        self.visit(node.elts)
+        self.to_add_set, temp = temp, self.to_add_set
+        self.to_add_set.append(tuple(temp))
 
     def visit_expr(self, node):
         assert self.gather_rvals
-        inner_symbols = []
-        with self.push_attributes(to_add_set=inner_symbols):
-            # call super generic_visit since self generic_visit calls visit_expr
-            super().generic_visit(node)
-        self.to_add_set.append(tuple(inner_symbols))
+        temp = self.to_add_set
+        self.to_add_set = []
+        # call super generic_visit since self generic_visit calls visit_expr
+        super().generic_visit(node)
+        self.to_add_set, temp = temp, self.to_add_set
+        self.to_add_set.append(tuple(temp))
 
     def generic_visit(self, node: 'Union[ast.AST, Sequence[ast.AST]]'):
         # The purpose of this is to make sure we call our visit_expr method if we see an expr
@@ -91,15 +99,30 @@ class GetAssignmentLvalRvalSymbolRefs(SaveOffAttributesMixin, VisitListsMixin, a
                 target_lval_symbols = []
                 with self.push_attributes(lval_symbols=target_lval_symbols):
                     self.visit(target)
-                self.lval_symbols.append(tuple(target_lval_symbols))
+                if isinstance(target, (ast.List, ast.Tuple)):
+                    # not strictly necessary since we are robust to this later,
+                    # but helps avoid unncessary double nesting, e.g., ((a, b, c),)
+                    assert len(target_lval_symbols) == 1
+                    assert isinstance(target_lval_symbols[0], tuple)
+                    self.lval_symbols.append(target_lval_symbols[0])
+                else:
+                    self.lval_symbols.append(tuple(target_lval_symbols))
         with self.gather_rvals_context():
             self.visit(node.value)
 
     def visit_Call(self, node):
+        extra_to_add = []
         if isinstance(node.func, (ast.Attribute, ast.Subscript)):
-            self.to_add_set.append(get_attrsub_symbol_chain(node))
+            extra_to_add.append(get_attrsub_symbol_chain(node))
+            to_visit = [node.args, node.keywords]
         else:
-            self.generic_visit(node)
+            to_visit = node
+        temp = self.to_add_set
+        self.to_add_set = []
+        self.generic_visit(to_visit)
+        self.to_add_set, temp = temp, self.to_add_set
+        temp = TiedTuple(set(_flatten(temp)) | set(_flatten(extra_to_add)))
+        self.to_add_set.append(temp)
 
     def visit_Attribute_or_Subscript(self, node):
         # TODO: we'll ignore args inside of inner calls, e.g. f.g(x, y).h
@@ -131,6 +154,27 @@ class GetAssignmentLvalRvalSymbolRefs(SaveOffAttributesMixin, VisitListsMixin, a
         # throw away anything appearing in lambda body that isn't bound
         self.rval_symbols = list(set(self.rval_symbols) - discard_set)
 
+    def visit_GeneratorExp(self, node):
+        self.visit_GeneratorExp_or_DictComp_or_ListComp_or_SetComp(node)
+
+    def visit_DictComp(self, node):
+        self.visit_GeneratorExp_or_DictComp_or_ListComp_or_SetComp(node)
+
+    def visit_ListComp(self, node):
+        self.visit_GeneratorExp_or_DictComp_or_ListComp_or_SetComp(node)
+
+    def visit_SetComp(self, node):
+        self.visit_GeneratorExp_or_DictComp_or_ListComp_or_SetComp(node)
+
+    def visit_GeneratorExp_or_DictComp_or_ListComp_or_SetComp(self, node):
+        assert self.gather_rvals
+        with self.push_attributes(rval_symbols=[]):
+            self.visit(node.generators)
+            discard_set = set(self.rval_symbols)
+        self.rval_symbols = list(set(self.rval_symbols) - discard_set)
+        # TODO: actually append the symbols that appear in the comprehension
+        self.rval_symbols.append(TiedTuple())
+
     def visit_arg(self, node):
         self.to_add_set.append(node.arg)
 
@@ -159,7 +203,10 @@ def _edges(lvals, rvals):
 
 
 def _edges_from_tuples(lvals, rvals):
-    if len(lvals) == len(rvals):
+    if isinstance(rvals, TiedTuple):
+        for lval in lvals:
+            yield from _edges(lval, rvals)
+    elif len(lvals) == len(rvals):
         for left, right in zip(lvals, rvals):
             yield from _edges(left, right)
     elif len(lvals) == 1:
