@@ -9,7 +9,7 @@ from .utils import retrieve_namespace_attr_or_sub
 
 if TYPE_CHECKING:
     from typing import Any, Optional, Set, Union
-    from .safety import DependencySafety
+    from .safety import NotebookSafety
     from .scope import Scope, NamespaceScope
 
 logger = logging.getLogger(__name__)
@@ -31,7 +31,7 @@ class DataSymbol(object):
             symbol_type: 'DataSymbolType',
             obj: 'Any',
             containing_scope: 'Scope',
-            safety: 'DependencySafety',
+            safety: 'NotebookSafety',
             parents: 'Optional[Set[DataSymbol]]' = None,
             refresh_cached_obj=False,
     ):
@@ -78,6 +78,9 @@ class DataSymbol(object):
     def __str__(self):
         return self.readable_name
 
+    def __hash__(self):
+        return hash(self.full_path)
+
     @property
     def is_subscript(self):
         return self.symbol_type == DataSymbolType.SUBSCRIPT
@@ -314,9 +317,12 @@ class DataSymbol(object):
                 #  namespace parents. Need a mechanism to mark for refresh without parent propagation.
                 self.safety.updated_symbols.add(alias)
         else:
-            for alias in self.safety.aliases[old_id]:
-                # print('propagate', updated_dep, 'to', alias, 'via', self, updated_dep.defined_cell_num, alias.defined_cell_num, self.defined_cell_num)
-                alias._propagate_update_to_deps(updated_dep, seen, parent_seen)
+            # propagating to all aliases was causing problems
+            # see test `test_class_redef`
+            self._propagate_update_to_deps(updated_dep, seen, parent_seen)
+            # for alias in self.safety.aliases[old_id]:
+            #     # print('propagate', updated_dep, 'to', alias, 'via', self, updated_dep.defined_cell_num, alias.defined_cell_num, self.defined_cell_num)
+            #     alias._propagate_update_to_deps(updated_dep, seen, parent_seen)
             if namespace is None and self.should_mark_stale(updated_dep):  # if we are at a leaf
                 self._propagate_update_to_namespace_parents(updated_dep, seen, parent_seen, refresh=refresh)
 
