@@ -6,7 +6,7 @@ import inspect
 import logging
 import re
 import sys
-from typing import TYPE_CHECKING
+from typing import cast, TYPE_CHECKING
 
 import black
 from IPython import get_ipython
@@ -50,7 +50,7 @@ def _safety_warning(node: 'DataSymbol'):
     )
 
 
-class DependencySafety(object):
+class NotebookSafety(object):
     """Holds all the state necessary to detect stale dependencies in Jupyter notebooks."""
     def __init__(self, cell_magic_name=None, use_comm=False, **kwargs):
         # Note: explicitly adding the types helps PyCharm's built-in code inspection
@@ -84,11 +84,15 @@ class DependencySafety(object):
         self._track_dependencies = True
 
         self._disable_level = 0
+        self._skip_unsafe_cells = kwargs.pop('skip_unsafe', True)
         self._prev_cell_nodes_with_stale_deps: Set[DataSymbol] = set()
 
         if self.use_comm:
             get_ipython().kernel.comm_manager.register_target(__package__, self._comm_target)
 
+    def set_active_cell(self, cell_id):
+        self._active_cell_id = cell_id
+
     def _comm_target(self, comm, open_msg):
         @comm.on_msg
         def _responder(msg):
@@ -99,11 +103,11 @@ class DependencySafety(object):
 
     def handle(self, request, comm=None):
         if request['type'] == 'change_active_cell':
-            cell_id = request['active_cell_id']
-            self._active_cell_id = cell_id
+            self.set_active_cell(request['active_cell_id'])
         elif request['type'] == 'cell_freshness':
-            cell_id = request['executed_cell_id']
-            self._counters_by_cell_id[cell_id] = self._last_execution_counter
+            cell_id = request.get('executed_cell_id', None)
+            if cell_id is not None:
+                self._counters_by_cell_id[cell_id] = self._last_execution_counter
             cells_by_id = request['content_by_cell_id']
             response = self.multicell_precheck(cells_by_id)
             response['type'] = 'cell_freshness'
@@ -116,28 +120,31 @@ class DependencySafety(object):
         stale_input_cells = []
         stale_output_cells = []
         fresh_cells = []
+        stale_symbols_by_cell_id: 'Dict[Union[int, str], Set[DataSymbol]]' = {}
+        killing_cell_ids_for_symbol: 'Dict[DataSymbol, Set[Union[int, str]]]' = defaultdict(set)
         for cell_id, cell_content in cells_by_id.items():
             try:
-                stale_nodes, max_defined_cell_num = self._precheck_stale_nodes(cell_content)
-                if len(stale_nodes) > 0:
+                stale_symbols, dead_symbols, max_defined_cell_num = self._precheck_stale_nodes(cell_content)
+                if len(stale_symbols) > 0:
+                    stale_symbols_by_cell_id[cell_id] = stale_symbols
                     stale_input_cells.append(cell_id)
-                elif max_defined_cell_num > self._counters_by_cell_id.get(cell_id, float('inf')):
-                    stale_output_cells.append(cell_id)
                 else:
-                    fresh_cells.append(cell_id)
+                    for dead_sym in dead_symbols:
+                        killing_cell_ids_for_symbol[dead_sym].add(cell_id)
+                    if max_defined_cell_num > self._counters_by_cell_id.get(cell_id, cast(int, float('inf'))):
+                        stale_output_cells.append(cell_id)
+                    else:
+                        fresh_cells.append(cell_id)
             except SyntaxError:
                 continue
         stale_links = defaultdict(list)
         refresher_links = defaultdict(list)
-        for candidate_refresher_cell_id in fresh_cells + stale_output_cells:
-            candidate_refresher_cell = cells_by_id[candidate_refresher_cell_id]
-            for stale_cell_id in stale_input_cells:
-                try:
-                    if not self._precheck_simple(f'{candidate_refresher_cell}\n{cells_by_id[stale_cell_id]}'):
-                        stale_links[stale_cell_id].append(candidate_refresher_cell_id)
-                        refresher_links[candidate_refresher_cell_id].append(stale_cell_id)
-                except SyntaxError:
-                    continue
+        for stale_cell_id in stale_input_cells:
+            stale_syms = stale_symbols_by_cell_id[stale_cell_id]
+            refresher_cell_ids = list(set.union(*(killing_cell_ids_for_symbol[stale_sym] for stale_sym in stale_syms)))
+            stale_links[stale_cell_id] = refresher_cell_ids
+            for refresher_cell_id in refresher_cell_ids:
+                refresher_links[refresher_cell_id].append(stale_cell_id)
         return {
             'stale_input_cells': stale_input_cells,
             'stale_output_cells': stale_output_cells,
@@ -156,34 +163,46 @@ class DependencySafety(object):
                 lines.append(line)
         return ast.parse('\n'.join(lines))
 
-    def compute_live_symbol_refs(self, code: 'Union[ast.Module, str]') -> 'Set[SymbolRef]':
+    def compute_live_dead_symbol_refs(self, code: 'Union[ast.Module, str]') -> 'Tuple[Set[SymbolRef], Set[SymbolRef]]':
         if isinstance(code, str):
             code = ast.parse(code)
         return ComputeLiveSymbolRefs(self)(code)
 
-    def _precheck_stale_nodes(self, cell: 'Union[ast.Module, str]'):
+    def _precheck_stale_nodes(self, cell: 'Union[ast.Module, str]') -> 'Tuple[Set[DataSymbol], Set[DataSymbol], int]':
         if isinstance(cell, str):
             cell = self._get_cell_ast(cell)
-        stale_nodes = set()
+        stale_symbols = set()
         max_defined_cell_num = -1
-        for symbol_ref in self.compute_live_symbol_refs(cell):
+        live_symbol_refs, dead_symbol_refs = self.compute_live_dead_symbol_refs(cell)
+        for symbol_ref in live_symbol_refs:
             if isinstance(symbol_ref, str):
-                node = self.global_scope.lookup_data_symbol_by_name_this_indentation(symbol_ref)
+                dsym = self.global_scope.lookup_data_symbol_by_name_this_indentation(symbol_ref)
             elif isinstance(symbol_ref, AttrSubSymbolChain):
-                node = self.global_scope.get_most_specific_data_symbol_for_attrsub_chain(symbol_ref, self.namespaces)
+                dsym = self.global_scope.get_most_specific_data_symbol_for_attrsub_chain(symbol_ref, self.namespaces)
             else:
                 logger.warning('invalid type for ref %s', symbol_ref)
                 continue
-            if node is None:
+            if dsym is None:
                 continue
             # print(node, node.has_stale_ancestor)
-            max_defined_cell_num = max(max_defined_cell_num, node.defined_cell_num)
-            if node.has_stale_ancestor:
-                stale_nodes.add(node)
-            if node.obj_id in self.namespaces:
-                namespace_scope = self.namespaces[node.obj_id]
+            max_defined_cell_num = max(max_defined_cell_num, dsym.defined_cell_num)
+            if dsym.has_stale_ancestor:
+                stale_symbols.add(dsym)
+            if dsym.obj_id in self.namespaces:
+                namespace_scope = self.namespaces[dsym.obj_id]
                 max_defined_cell_num = max(max_defined_cell_num, namespace_scope.max_defined_timestamp)
-        return stale_nodes, max_defined_cell_num
+        dead_symbols = set()
+        for symbol_ref in dead_symbol_refs:
+            if isinstance(symbol_ref, str):
+                dsym = self.global_scope.lookup_data_symbol_by_name_this_indentation(symbol_ref)
+            elif isinstance(symbol_ref, AttrSubSymbolChain):
+                dsym = self.global_scope.get_most_specific_data_symbol_for_attrsub_chain(symbol_ref, self.namespaces)
+            else:
+                logger.warning('invalid type for ref %s', symbol_ref)
+                continue
+            if dsym is not None:
+                dead_symbols.add(dsym)
+        return stale_symbols, dead_symbols, max_defined_cell_num
 
     def _precheck_simple(self, cell):
         return len(self._precheck_stale_nodes(cell)[0]) > 0
@@ -197,7 +216,7 @@ class DependencySafety(object):
             return False
         self.statement_cache[cell_counter()] = compute_lineno_to_stmt_mapping(cell_ast)
         if self._last_refused_code is None or cell != self._last_refused_code:
-            self._prev_cell_nodes_with_stale_deps, _ = self._precheck_stale_nodes(cell_ast)
+            self._prev_cell_nodes_with_stale_deps = self._precheck_stale_nodes(cell_ast)[0]
             if len(self._prev_cell_nodes_with_stale_deps) > 0 and self._disable_level < 2:
                 warning_counter = 0
                 for node in self._prev_cell_nodes_with_stale_deps:
@@ -245,7 +264,7 @@ class DependencySafety(object):
                 self._counters_by_cell_id[self._active_cell_id] = self._last_execution_counter
                 self._active_cell_id = None
             # Stage 1: Precheck.
-            if self._precheck_for_stale(cell):
+            if self._precheck_for_stale(cell) and self._skip_unsafe_cells:
                 # FIXME: hack to increase cell number
                 #  ideally we shouldn't show a cell number at all if we fail precheck since nothing executed
                 return run_cell_func('None')
