@@ -16,7 +16,7 @@ from .ipython_utils import cell_counter
 
 if TYPE_CHECKING:
     from typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Union
-    from .safety import DependencySafety
+    from .safety import NotebookSafety
 
 
 class Scope(object):
@@ -24,7 +24,7 @@ class Scope(object):
 
     def __init__(
             self,
-            safety: 'DependencySafety',
+            safety: 'NotebookSafety',
             scope_name: str = GLOBAL_SCOPE_NAME,
             parent_scope: 'Optional[Scope]' = None,
     ):
@@ -110,7 +110,11 @@ class Scope(object):
         for name in chain.symbols:
             if isinstance(name, CallPoint):
                 return dsym
-            dsym = cur_scope.lookup_data_symbol_by_name_this_indentation(name)
+            next_dsym = cur_scope.lookup_data_symbol_by_name_this_indentation(name)
+            # HUGE HACK: prevents us from checking namespace symbols unless entire namespace is stale
+            # TODO: get rid of the cell number check once namespace symbols created for dictionary literals
+            if next_dsym is not None or dsym is None or dsym.defined_cell_num >= dsym.required_cell_num:
+                dsym = next_dsym
             if name_to_obj is None:
                 break
             try:
