@@ -11,20 +11,21 @@ if TYPE_CHECKING:
     from types import FrameType
     from typing import List, Optional, Set
     from ..data_symbol import DataSymbol
-    from ..safety import DependencySafety
+    from ..safety import NotebookSafety
     from ..scope import Scope, NamespaceScope
 
 logger = logging.getLogger(__name__)
 
 
 class TraceStatement(object):
-    def __init__(self, safety: 'DependencySafety', frame: 'FrameType', stmt_node: 'ast.stmt', scope: 'Scope'):
+    def __init__(self, safety: 'NotebookSafety', frame: 'FrameType', stmt_node: 'ast.stmt', scope: 'Scope'):
         self.safety = safety
         self.frame = frame
         self.stmt_node = stmt_node
         self.scope = scope
         self.class_scope: Optional[NamespaceScope] = None
         self.call_point_deps: List[Set[DataSymbol]] = []
+        self.lambda_call_point_deps_done_once = False
         self._marked_finished = False
 
     @contextmanager
@@ -112,7 +113,6 @@ class TraceStatement(object):
                 rval_symbol_refs=set.union(*symbol_edges.values()) - {None}
             ) | deep_rval_deps
         for scope, obj, attr_or_sub, is_subscript in self.safety.attr_trace_manager.saved_store_data:
-            # print(scope, obj, attr_or_sub, is_subscript)
             try:
                 attr_or_sub_obj = retrieve_namespace_attr_or_sub(obj, attr_or_sub, is_subscript)
             except (AttributeError, KeyError, IndexError):
