@@ -85,6 +85,19 @@ def test_long_chain():
     assert_detected('f has stale dependency on old value of a')
 
 
+@skipif_known_failing
+def test_for_loop_liveness_check():
+    run_cell('x = 0')
+    run_cell('y = x + 1')
+    run_cell('x = 42')
+    run_cell("""
+for i in range(2):
+    logging.info(y)
+    y = 7
+""")
+    assert_detected('first use of `y` in loop has stale dependency on old value of `x`')
+
+
 def test_redef_after_stale_use():
     run_cell('a = 1')
     run_cell('b = a + 1')
@@ -96,6 +109,19 @@ b = 7
     assert_detected('b has stale dependency on old value of a')
 
 
+def test_class_redef():
+    classdef = """
+class Foo:
+    def __init__(self, x):
+        self.x = x"""
+    run_cell(classdef)
+    run_cell('x = 5')
+    run_cell('foo = Foo(x)')
+    run_cell(classdef)
+    run_cell('logging.info(x)')
+    assert_not_detected('`x` independent of class `Foo`')
+
+
 def test_subscript_dependency_fp():
     run_cell('lst = [0, 1, 2]')
     run_cell('x = 5')
@@ -105,6 +131,40 @@ def test_subscript_dependency_fp():
     assert_not_detected('y depends only on unchanged lst[0] and not on changed lst[1]')
 
 
+def test_comprehension_generator_vars_not_live():
+    run_cell('x = 0')
+    run_cell('y = x + 6')
+    run_cell('x = 42')
+    run_cell('lst = [y for y in range(1) for j in range(1)]')
+    assert_not_detected('`y` is not live in the list comprehension')
+
+
+def test_lambda_params_not_live():
+    run_cell('x = 0')
+    run_cell('y = x + 6')
+    run_cell('x = 42')
+    run_cell('f = lambda y: y * 2')
+    assert_not_detected('`y` is not live in the lambda expr')
+
+
+def test_fundef_params_not_live():
+    run_cell('x = 0')
+    run_cell('y = x + 6')
+    run_cell('x = 42')
+    run_cell("""
+def f(y):
+    return y + 9""")
+    assert_not_detected('`y` is not live in the function')
+
+
+def test_no_edge_for_comprehension_variable():
+    run_cell('i = 0')
+    run_cell('x = [i for i in range(10)]')
+    run_cell('i = 1')
+    run_cell('logging.info(x)')
+    assert_not_detected('`x` does not depend on `i`')
+
+
 # simple test about the basic assignment
 def test_basic_assignment():
     run_cell('a = 1')
@@ -762,6 +822,26 @@ def test_pandas_attr_mutation_with_alias():
                     'which in general could be dangerous')
 
 
+def test_stale_detection_works_when_namespace_available_but_stale_symbol_unavailable():
+    run_cell('import pandas as pd')
+    run_cell('data = {"a": list(range(5)), "b": list(range(1, 6))}')
+    run_cell('df = pd.DataFrame(data)')
+    run_cell('df.b')  # force creation of a namespace for df
+    run_cell('data = {"a": list(range(4)), "b": list(range(1, 5))}')
+    run_cell('logging.info(df.a)')
+    assert_detected('`df.a` depends on stale `data` dictionary')
+
+
+def test_stale_detection_works_when_namespace_available_but_stale_symbol_unavailable_2():
+    run_cell('import pandas as pd')
+    run_cell('data = {"a": list(range(5)), "b": list(range(1, 6))}')
+    run_cell('df = pd.DataFrame(data)')
+    run_cell('df.b')  # force creation of a namespace for df
+    run_cell('data["b"] = 42')
+    run_cell('logging.info(df.a)')
+    assert_false_positive('`df.a` independent of entry `data["b"]`')
+
+
 def test_list_alias_breaking():
     run_cell('x = [0]')
     run_cell('y = x')
